/* NO USER CONFIGURABLE PARAMETERS HERE! */

/* Path Integral Monte Carlo code developed by A. Balaz
   (antun@ipb.ac.rs) for the paper:

   "Systematic Speedup in the Convergence of Path Integrals"
   by A. Bogojevic, A. Balaz, and A. Belic
   Phys. Rev. Lett. 94 (2005) 180403

   Address: Institute of Physics, Belgrade, Serbia
   Scientific Computing Laboratory, http://www.scl.rs/speedup/

   Public use and modification of this code is allowed providing
   the above paper is properly acknowledged. The author would be
   grateful for all information and/or comments regarding the use
   of the code.
*/

/* This is the header for file p.c. */

#include <math.h>

/* Parameters of the potential, defined in main(), used in V0(). */
extern double *par;

/* Array with powers of 2 (two[n] = 2^n), defined in main(), used in func(). */
extern int *two;

/* Array of path integral norms, defined in main(), used in func(). */
extern double *norm;

/* Array of the exponents of Gaussian probability distribution functions,
   defined in distr(), used in func(). */
extern double *distrexp;

/* Variables used for storing the values of the (unimproved) potential V (V00) and its derivatives
   (V01, ..., V34). They are here initialized to zero, and later changed (only the nonzero ones!)
   by the user supplied function V0().
*/
double V00 = 0, V01 = 0, V02 = 0, V03 = 0, V04 = 0, V05 = 0, V06 = 0, V07 = 0, V08 = 0, V09 = 0;
double V10 = 0, V11 = 0, V12 = 0, V13 = 0, V14 = 0, V15 = 0, V16 = 0, V17 = 0, V18 = 0, V19 = 0;
double V20 = 0, V21 = 0, V22 = 0, V23 = 0, V24 = 0, V25 = 0, V26 = 0, V27 = 0, V28 = 0, V29 = 0;
double V30 = 0, V31 = 0, V32 = 0, V33 = 0, V34 = 0;

/* Variables used in function Vp(), for optimization purposes
   (powers of the derivatives of the potential, powers of delta, and powers of eps = T / N)
*/
double V012, V013, V014, V015, V016, V017, V018, V022, V023, V024, V025, V026, V027, V028, V029;
double V032, V033, V034, V035, V036, V042, V043, V044, V045, V046, V052, V053, V054, V062, V063, V064;
double V072, V073, V074, V082, V083, V092, V093, V102, V103, V112, V122, V132, V142, V152, V162;
double delta, delta2, delta4, delta6, delta8, delta10, delta12, delta14, delta16, delta18, delta20, delta22, delta24, delta26, delta28, delta30, delta32, delta34;
double eps, epsinv, eps2, eps3, eps4, eps5, eps6, eps7, eps8, eps9, eps10, eps11, eps12, eps13, eps14, eps15, eps16, eps17;

/* Variables used in the function func(). The variable qbar is the mid-point for a pair of points
   from the chosen path, variable action is the action for the given trajectory.
*/
double qbar, action;

/* Variables used in function func(). Variable Nmax = 2^s is the maximal number of time steps.
   Variable step is equal to 2^(s - n). In the bisection algorithm with Nmax = 2^s, paths are
   given by the arrays (q[0], ..., q[N]). If the current number of time steps is Nc = 2^n, the
   relevant paths are (q[0], q[step], ..., q[2^n step = Nmax]). 
*/
int Nmax, Nc, step;
